'GenCalc - Genealogy Date Calculator
'By Charles Dietz 9/25/2007
'Written for Carol Tefft to simplify genealogy event calculations
'==========================================================================================
'ver 1.10
'  - corrected problem with Enter/click Ok differences (added set focus stmt)
'  - added new age feature '+' to years (such as 60+ years)
'ver 1.30 (12-20-07)
'  - added calendar
'  - added week day checkbox
'ver 1.40 (12-22-07)
'  - 4 digit error msg displayed repeatedly for week day option
'ver 1.50 (05-17-08)
'  - simplified and corrected calcDate() algorithm
'  - keep focus same as before help and calendar clicks
'  - add dummy textbox as first textbox to maintain consistant hiLite and cursor pos
'  - warning if illegal date is entered
'  - hiLites date being copied with [Cp]
'ver 1.51 (05-30-08)
'  - corrected spelling of 'genealogy'
'ver 1.52 (04-25-11)
'  - corrected minor errors associated with dates
'  - can now allow date as: "ap 20,   1888" --> "04/20/1888"
'  - eliminated repeated error messages when calculating with less than two inputs
'  - corrected error calculating associated with "months abbreviated" checkbox
'=====================================================================================

#COMPILE EXE
#DIM ALL
#RESOURCE "GenCalc.pbr"
'#INCLUDE "win32api.inc"
'#INCLUDE "CommCtrl.inc"
#INCLUDE "GenCalcAPI.inc"
'#INCLUDE "debug.inc"

$ver = "1.52"
%IDC_Dummy  = 100
%IDC_Event  = 101
%IDC_Years  = 102
%IDC_Months = 103
%IDC_Days   = 104
%IDC_Birth  = 105
%IDC_Format = 106
%IDC_WkDay  = 107
%IDC_Calc   = 108
%IDC_Reset  = 111
%IDC_Copy1  = 112
%IDC_Copy2  = 113
%IDC_Cal1   = 114
%IDC_Cal2   = 115
%IDC_Help1  = 121
%IDC_Help2  = 122
%IDC_Help3  = 123
%IDC_Help4  = 124
%IDC_Help5  = 125
%USEMACROS = 1

GLOBAL hMain, calcMode, noDays AS LONG
GLOBAL hCal, hdc, hFont, hFont1, lenDay, top AS LONG
GLOBAL xCal, yCal, calDay, nErr, noEnter AS LONG
GLOBAL months() AS STRING

'DECLARE FUNCTION WindowMessage LIB "WINMSG.DLL" ALIAS "WindowMessageA" (BYVAL MsgNum AS LONG) AS STRING

#INCLUDE "myMsgbox.inc"

MACRO setSubclass(hDlg, hCtl, proc)
    SetWindowLong GetDlgItem(hDlg, hCtl), %GWL_USERDATA, _
        SetWindowLong(GetDlgItem(hDlg, hCtl), %GWL_WNDPROC, CODEPTR(proc))
END MACRO
MACRO killSubclass(hCtl)
    SetWindowLong(GetDlgItem(CBHNDL, hCtl), %GWL_WNDPROC, GetWindowLong(CBHNDL, %GWL_USERDATA))
END MACRO
MACRO setOrigProc
    FUNCTION = CallWindowProc(GetWindowLong(hWnd&, %GWL_USERDATA), hWnd&, wMsg&, wParm&, lParm&)
END MACRO

FUNCTION PBMAIN()
   LOCAL nStyle, h, v AS LONG
   nStyle = %WS_CAPTION OR %DS_MODALFRAME OR %WS_SYSMENU OR %WS_MINIMIZEBOX
   DIALOG NEW PIXELS, 0, "Genealogy Date Calculator", , , 295, 385, nStyle TO hMain
   CONTROL ADD TEXTBOX, hMain, %IDC_Dummy, "", 0, 0, 0, 0
   CONTROL ADD LABEL, hMain, -1, "Date of event:", 10, 20, 90, 16
   CONTROL ADD TEXTBOX, hMain, %IDC_Event, "", 80, 20, 120, 20
   h = 40: v = 60
   CONTROL ADD FRAME, hMain, -1, "Age", h, v, 160, 140
   CONTROL ADD LABEL, hMain, -1, "Years:", h + 20, v + 30, 40, 16
   CONTROL ADD TEXTBOX, hMain, %IDC_Years, "", h + 65, v + 30, 70, 20
   CONTROL ADD LABEL, hMain, -1, "Months:", h + 20, v + 60, 40, 16
   CONTROL ADD TEXTBOX, hMain, %IDC_Months, "", h + 65, v + 60, 70, 20
   CONTROL ADD LABEL, hMain, -1, "Days:", h + 20, v + 90, 40, 16
   CONTROL ADD TEXTBOX, hMain, %IDC_Days, "", h + 65, v + 90, 70, 20

   CONTROL ADD LABEL, hMain, -1, "Date of birth:", 10, v + 170, 70, 16
   CONTROL ADD TEXTBOX, hMain, %IDC_Birth, "", 80, v + 170, 120, 20

   nStyle = %BS_NOTIFY OR %WS_TABSTOP
   CONTROL ADD BUTTON, hMain, %IDC_Calc, "&Calc", 56, v + 220, 60, 30, nStyle
   CONTROL ADD BUTTON, hMain, %IDC_Reset, "&Reset", 118, v + 220, 60, 30
   CONTROL ADD BUTTON, hMain, %IDCANCEL, "C&ancel", 180, v + 220, 60, 30
   CONTROL ADD CHECKBOX, hMain, %IDC_Format, "Show month abbreviated", 56, v + 270, 150, 20, nStyle
   CONTROL ADD CHECKBOX, hMain, %IDC_WkDay, "Show week day", 56, v + 295, 150, 20, nStyle

   CONTROL ADD IMGBUTTONX, hMain, %IDC_Cal1, "calendar", 215, 20, 20, 20
   CONTROL ADD BUTTON, hMain, %IDC_Copy1, "Cp", 240, 20, 20, 20
   CONTROL ADD BUTTON, hMain, %IDC_Help1, "?", 265, 20, 20, 20
   CONTROL ADD IMGBUTTONX, hMain, %IDC_Cal2, "calendar", 215, v + 170, 20, 20
   CONTROL ADD BUTTON, hMain, %IDC_Copy2, "Cp", 240, v + 170, 20, 20
   CONTROL ADD BUTTON, hMain, %IDC_Help5, "?", 265, v + 170, 20, 20
   CONTROL ADD BUTTON, hMain, %IDC_Help2, "?", 220, v + 60, 20, 20
   CONTROL ADD BUTTON, hMain, %IDC_Help3, "?", 220, v + 270, 20, 20
   CONTROL ADD BUTTON, hMain, %IDC_Help4, "?", 220, v + 295, 20, 20

   DIALOG SHOW MODAL hMain CALL mainProc
   END FUNCTION

CALLBACK FUNCTION mainProc()
   LOCAL i, m, n AS LONG, s AS STRING
   STATIC ctrl, hCtrl AS LONG
   DIM sErr(1 TO 4) AS STATIC STRING
   SELECT CASE CBMSG
      'insure that winMsg.dll is in winAPI folder and declare WindowMessage
      'OPEN "log.txt" FOR APPEND AS #1
      'PRINT #1, WindowMessage(CBMSG), TIME$, STR$(CBCTL), STR$(CBWPARAM), STR$(CBLPARAM)
      'CLOSE #1
      CASE %WM_INITDIALOG
         'setup subclasses
         FOR i = %IDC_Event TO %IDC_Birth
            setSubclass(CBHNDL, i, EditSubClassProc)
         NEXT i
         sErr(1) = "months": sErr(2) = "days": sErr(3) = "years": sErr(4) = "4 digit year"
         CONTROL SET FOCUS CBHNDL, %IDC_Event
      CASE %WM_PAINT
      CASE %WM_COMMAND
         SELECT CASE CBCTL
            CASE %IDC_Dummy
               IF CBCTLMSG = %EN_SETFOCUS THEN
                  CONTROL SET FOCUS CBHNDL, %IDC_Event
               END IF
            CASE %IDC_Event TO %IDC_Calc
               IF CBCTLMSG = %EN_KILLFOCUS AND (CBCTL = %IDC_Event OR CBCTL = %IDC_Birth) THEN
                  nErr = 0: IF ctrl = 0 THEN ctrl = CBCTL
               END IF
               IF CBCTLMSG = %EN_SETFOCUS OR CBCTLMSG = %BN_SETFOCUS THEN
                  IF nErr = 0 AND ctrl AND CBCTL <> ctrl THEN
                     showDateFmt(ctrl)
                     IF nErr THEN
                        myMSGBOX "Check on " + sErr(nErr), %MB_ICONERROR, "Illegal Date"
                        CONTROL SET FOCUS CBHNDL, ctrl
                     ELSE
                        ctrl = 0
                     END IF
                  END IF
               END IF
               IF (CBCTL = %IDC_Format OR CBCTL = %IDC_WkDay) AND CB.CTLMSG = %BN_CLICKED THEN
                  IF nErr = 0 THEN
                     showDateFmt(%IDC_Event)
                     showDateFmt(%IDC_Birth)
                  END IF
               ELSEIF CBCTL = %IDC_Calc AND CBCTLMSG = %BN_SETFOCUS THEN
                  IF nErr = 0 THEN
                     calculate: noDays = 0
                     IF calcMode = 1 THEN
                        showDateFmt(%IDC_Event)
                     ELSEIF calcMode = 3 THEN
                        showDateFmt(%IDC_Birth)
                     END IF
                  END IF
               END IF
            CASE %IDC_Cal1, %IDC_Cal2
               m = INSTR(s, "["): IF m THEN s = TRIM$(LEFT$(s, m - 1))
               m = PARSECOUNT(s, "/"): n = 0
               IF m = 1 THEN
                  n = VAL(s): IF n < 1600 OR n > 2100 THEN n = 0 ELSE n = n * 10000
               ELSEIF m = 2 THEN
                  n = 10000 * VAL(PARSE$(s, "/", 2)) + 100 * VAL(PARSE$(s, "/", 1))
               ELSEIF m = 3 THEN
                  n = 10000 * VAL(PARSE$(s, "/", 3)) + 100 * VAL(PARSE$(s, "/", 1)) + VAL(PARSE$(s, "/", 2))
               END IF
               s = calendar(n)
               IF LEN(s) THEN
                  IF CBCTL = %IDC_Cal1 THEN
                     CONTROL SET TEXT CBHNDL, %IDC_Event, s
                     CONTROL SET FOCUS CBHNDL, %IDC_Event
                  ELSE
                     CONTROL SET TEXT CBHNDL, %IDC_Birth, s
                     CONTROL SET FOCUS CBHNDL, %IDC_Birth
                  END IF
               END IF
               CONTROL SET FOCUS CBHNDL, ctrl
            CASE %IDC_Copy1, %IDC_Copy2
               IF CBCTL = %IDC_Copy1 THEN n = %IDC_Event ELSE n = %IDC_Birth
               CONTROL SET FOCUS CBHNDL, n
               CONTROL GET TEXT CBHNDL, n TO s
               IF LEN(s) THEN
                  CONTROL SEND CBHNDL, n, %EM_SETSEL, 0, -1
                  Clipboard_SetText(s)
               END IF
            CASE %IDC_Help1, %IDC_Help5
               s = " Enter event date as: " + $CRLF _
                 + "    mm/dd/yyyy" + $CRLF _
                 + "    or mm/yyyy" + $CRLF _
                 + "    or yyyy" + $CRLF + $CRLF _
                 + " Some possible events include death, marriage, oath signing " + $CRLF _
                 + " enlistments, census year or census taker's date." + $CRLF + $CRLF _
                 + " The calendar button displays a 12 month calendar and the  " + $CRLF _
                 + " copy button (Cp) can be used to copy text to the clipboard."
               myMsgbox s, %MB_ICONINFORMATION, "Ver " + $ver
               IF CBCTL = %IDC_Help1 THEN ctrl = %IDC_Event ELSE ctrl = %IDC_Birth
               CONTROL SET FOCUS CBHNDL, ctrl
            CASE %IDC_Help2
               s = " Any combination of years, months, and days can be specified   " + $CRLF + $CRLF _
                 + " You may enter an age range such as 20-30 years,   " + $CRLF _
                 + " or -5 (for 5 and under), or 60+ (for 60 and over)."
               myMsgbox s, %MB_ICONINFORMATION, "Ver " + $ver
               CONTROL SET FOCUS CBHNDL, ctrl
            CASE %IDC_Help3
               s = "Changes date format from numerical display to   " + $CRLF _
                 + "3-character month abbreviation, or visa versa   " + $CRLF _
                 + "e.g.,  Dec 07, 1941"
               myMsgbox s, %MB_ICONINFORMATION, "Ver " + $ver
               CONTROL SET FOCUS CBHNDL, ctrl
            CASE %IDC_Help4
               s = "show day of week with event and birth dates in brackets   "  + $CRLF _
                 + "e.g.  Dec 07, 1941 [Su]   "
               myMsgbox s, %MB_ICONINFORMATION, "Ver " + $ver
               CONTROL SET FOCUS CBHNDL, ctrl
            CASE %IDC_Reset
               FOR i = 1 TO 5
                  CONTROL SET TEXT CBHNDL, 100 + i, ""
               NEXT i
               calcMode = 0: noDays = 0
               CONTROL SET FOCUS CBHNDL, %IDC_Event
               CONTROL SET CHECK CBHNDL, %IDC_Format, 0
               CONTROL SET CHECK CBHNDL, %IDC_WkDay, 0
            CASE %IDCANCEL
               DIALOG END CBHNDL
         END SELECT
         IF CBCTL = %IDC_Calc OR CBCTL = %IDC_Reset THEN
            noEnter = 1
            CONTROL SEND CBHNDL, CBCTL, %BM_SETSTYLE, %BS_PUSHBUTTON, %TRUE
         ELSEIF CBCTL >= %IDC_Help1 AND CBCTL <= %IDC_Help5 THEN
            noEnter = 1
         ELSE
            noEnter = 0
         END IF
      CASE %WM_SYSCOMMAND
      CASE %WM_DESTROY
         FOR i = %IDC_Event TO %IDC_Birth
            killSubclass(i)
         NEXT i
   END SELECT
END FUNCTION

FUNCTION EditSubClassProc(BYVAL hWnd&, BYVAL wMsg&, BYVAL wParm&, BYVAL lParm&) AS LONG
   'capture Return key
   SELECT CASE wMsg&
      CASE %WM_KEYUP
         IF wParm& = 13 THEN
            IF noEnter THEN EXIT FUNCTION
            SetFocus GetNextDlgTabItem(hMain, GetFocus(), %FALSE)
         END IF
   END SELECT
   setOrigProc   'pass the message on to the original window procedure
END FUNCTION

FUNCTION daysInMonth(mo AS LONG, yr AS LONG) AS LONG
   LOCAL days AS INTEGER
   IF mo < 8 THEN
      days = 30 + mo MOD 2
   ELSE
      days = 30 + (mo + 1) MOD 2
   END IF
   IF mo = 2 THEN
      days = 28
      IF (yr MOD 100 <> 0 OR yr MOD 400 = 0) _
         AND yr MOD 4 = 0 THEN INCR days 'leap yr
   END IF
   daysInMonth = days
END FUNCTION

SUB processDate(s AS STRING)
   LOCAL sm, sd, sy, month, ss, sWkDay AS STRING
   LOCAL n AS LONG
   month = "Jan, Feb, Mar, Apr, May, Jun, Jul, Aug, Sep, Oct, Nov, Dec"
   n = INSTR(s, "[")
   IF n THEN
      sWkDay = MID$(s, n): s = TRIM$(LEFT$(s, n - 1))
   END IF
   IF LEN(s) THEN
      WHILE INSTR(s, "  ") 'eliminate multiple spaces
         REPLACE "  " WITH " " IN s
      WEND
      ss = PARSE$(s, ANY " ,", 1)
      n = INSTR(LCASE$(month), LCASE$(ss))
      IF n THEN
         REPLACE ss WITH TRIM$(STR$((n-1)\5+1)) IN s
         REPLACE ", " WITH " " IN s
         REPLACE "," WITH " " IN s
         REPLACE " " WITH "/" IN s
      END IF
      REPLACE "-" WITH "/" IN s
      IF INSTR(s, "to") OR INSTR(s, "or") THEN
         sy = SPACE$(4)
      ELSEIF PARSECOUNT(s, "/") = 1 THEN
         sy = s
      ELSEIF PARSECOUNT(s, "/") = 2 THEN
         sm = PARSE$(s, "/", 1): sy = PARSE$(s, "/", 2)
         sm = RIGHT$("0" + sm, 2): noDays = 1: s = sm + "/" + sy
      ELSE
         sm = PARSE$(s, "/", 1): sd = PARSE$(s, "/", 2): sy = PARSE$(s, "/", 3)
         sm = RIGHT$("0" + sm, 2): sd = RIGHT$("0" + sd, 2)
         s = sm + "/" + sd + "/" + sy
      END IF
      IF LEN(sy) <> 4 THEN nErr = 4: sWkDay = "": EXIT SUB
      IF LEN(sWkDay) THEN s = s + "  " + sWkDay
   END IF
END SUB

SUB calculate()
   LOCAL sEvent, sYears, sMonths, sDays, sBirth, s AS STRING
   LOCAL nTime, m, d, y, mm, dd, yr, yr0, yr1, yr2, u, v, w, x AS LONG
   nErr = 0
   CONTROL GET TEXT hMain, %IDC_Event TO sEvent
   CONTROL GET TEXT hMain, %IDC_Years TO sYears
   CONTROL GET TEXT hMain, %IDC_Months TO sMonths
   CONTROL GET TEXT hMain, %IDC_Days TO sDays
   CONTROL GET TEXT hMain, %IDC_Birth TO sBirth
   sEvent = TRIM$(PARSE$(sEvent, "[", 1))
   sBirth = TRIM$(PARSE$(sBirth, "[", 1))
   nTime = LEN(sYears) + LEN(sMonths) + LEN(sDays)
   IF LEN(sEvent) AND LEN(sBirth) AND nTime = 0 THEN calcMode = 2
   IF LEN(sEvent) AND nTime AND LEN(sBirth) = 0 THEN calcMode = 3
   IF LEN(sBirth) AND nTime AND LEN(sEvent) = 0 THEN calcMode = 1
   IF calcMode = 2 THEN 'calc age
      IF LEN(sBirth) THEN processDate(sBirth)
      s = REMOVE$(sBirth, "/"): u = VAL(RIGHT$(s, 4) + LEFT$(s, 4))
      IF LEN(sEvent) THEN processDate(sEvent)
      s = REMOVE$(sEvent, "/"): v = VAL(RIGHT$(s, 4) + LEFT$(s, 4))
      IF u > v THEN
         MSGBOX "Event date before birth date", %MB_ICONERROR, "Data Error"
         CONTROL SET FOCUS hMain, %IDC_Event: EXIT SUB
      END IF
      m = VAL(LEFT$(sBirth, 2)): d = VAL(MID$(sBirth, 4, 2)): y = VAL(RIGHT$(sBirth, 4))
      mm = VAL(LEFT$(sEvent, 2)): dd = VAL(MID$(sEvent, 4, 2)): yr = VAL(RIGHT$(sEvent, 4))
      u = mm - m: v = dd - d: w = yr - y
      IF PARSECOUNT(sEvent, "/") = 1 OR PARSECOUNT(sBirth, "/") = 1 THEN x = 1
      IF x THEN
         s = STR$(w) + " or" + STR$(w - 1)
         CONTROL SET TEXT hMain, %IDC_Years, s
         CONTROL SET TEXT hMain, %IDC_Months, ""
         CONTROL SET TEXT hMain, %IDC_Days, ""
         EXIT SUB
      ELSE
         IF v < 0 THEN DECR u: v = v + daysInMonth(m + u, y + w)
         IF u < 0 THEN DECR w: u = u + 12
         CONTROL SET TEXT hMain, %IDC_Years, STR$(w)
         CONTROL SET TEXT hMain, %IDC_Months, STR$(u)
         IF noDays THEN s = "" ELSE s = STR$(v)
         CONTROL SET TEXT hMain, %IDC_Days, s
      END IF
   ELSEIF calcMode = 3 THEN 'calc date of birth
      IF LEN(sEvent) THEN processDate(sEvent)
      yr = VAL(sYears): mm = VAL(sMonths): dd = VAL(sDays)
      IF INSTR(sYears, "-") THEN
         yr0 = VAL(PARSE$(sYears, "-", 1)): yr1 = VAL(PARSE$(sYears, "-", 2))
         CONTROL SET TEXT hMain, %IDC_Months, "": sMonths = ""
         CONTROL SET TEXT hMain, %IDC_Days, "": sDays = ""
      ELSEIF INSTR(sYears, "+") THEN
         yr2 = VAL(PARSE$(sYears, "+", 1))
         CONTROL SET TEXT hMain, %IDC_Months, "": sMonths = ""
         CONTROL SET TEXT hMain, %IDC_Days, "": sDays = ""
      END IF
      IF yr0 + yr1 THEN yr = yr0
      IF yr2 THEN yr = yr2
      s = calcDate(sEvent, mm, dd, yr, -1) 'calc date of birth
      IF (LEN(sMonths) = 0 AND LEN(sDays) = 0) OR PARSECOUNT(sEvent, "/") = 1 THEN
         y = VAL(RIGHT$(s, 4))
         s = STR$(y) + " or" + STR$(y - 1)
         IF yr0 + yr1 THEN
            s = STR$(y - yr1 + yr0 - 1) + " to" + STR$(y)
         ELSEIF yr2 THEN
            s = STR$(y) + " or before"
         END IF
      END IF
      CONTROL SET TEXT hMain, %IDC_Birth, ""
      CONTROL SET TEXT hMain, %IDC_Birth, TRIM$(s)
   ELSEIF calcMode = 1 THEN 'calc date of event
      IF LEN(sBirth) THEN processDate(sBirth)
      yr = VAL(sYears): mm = VAL(sMonths): dd = VAL(sDays)
      IF INSTR(sYears, "-") THEN
         yr0 = VAL(PARSE$(sYears, "-", 1)): yr1 = VAL(PARSE$(sYears, "-", 2))
         CONTROL SET TEXT hMain, %IDC_Months, "": sMonths = ""
         CONTROL SET TEXT hMain, %IDC_Days, "": sDays = ""
      ELSEIF INSTR(sYears, "+") THEN
         yr2 = VAL(PARSE$(sYears, "+", 1))
         CONTROL SET TEXT hMain, %IDC_Months, "": sMonths = ""
         CONTROL SET TEXT hMain, %IDC_Days, "": sDays = ""
      END IF
      IF yr0 + yr1 THEN yr = yr0
      IF yr2 THEN yr = yr2
      s = calcDate(sBirth, mm, dd, yr, 1) 'date of event
      IF (LEN(sMonths) = 0 AND LEN(sDays) = 0) OR PARSECOUNT(sBirth, "/") = 1 THEN
         y = VAL(RIGHT$(s, 4))
         s = STR$(y) + " or" + STR$(y + 1)
         IF yr0 + yr1 THEN
            s = STR$(y) + " to" + STR$(y + yr1 - yr0 + 1)
         ELSEIF yr2 THEN
            s = STR$(y) + " or later"
         END IF
      END IF
      CONTROL SET TEXT hMain, %IDC_Event, ""
      CONTROL SET TEXT hMain, %IDC_Event, TRIM$(s)
   ELSE
      nErr = 1
      MSGBOX "Must specify two of the three data parts", %MB_ICONERROR, "Data error"
      CONTROL SET FOCUS hMain, %IDC_Event
   END IF
END SUB

FUNCTION calcDate(sDate AS STRING, u AS LONG, v AS LONG, w AS LONG, dir AS LONG) AS STRING
   LOCAL m, d, y, mo, dd, yr, daysInMo AS LONG
   LOCAL sMonth, sDay, sYear AS STRING
   m = u: d = v: y = w
   mo = VAL(LEFT$(sDate, 2)): yr = VAL(RIGHT$(sDate, 4))
   IF noDays = 0 THEN dd = VAL(MID$(sDate, 4, 2))
'   m = m + 12 * y
   IF dir < 0 THEN 'decreasing time span
      yr = yr - y
      mo = mo - m
      dd = dd - d
      IF mo < 0 THEN mo = 12 + mo: DECR yr
      IF dd < 0 THEN
         dd = dd + daysInMonth(mo, yr): DECR mo
         IF mo < 1 THEN mo = mo + 12: DECR yr
      END IF
   ELSE 'increasing time span
      yr = yr + y
      mo = mo + m
      dd = dd + d
      IF mo > 12 THEN mo = mo - 12: INCR yr
      daysInMo = daysInMonth(mo, yr)
      IF dd > daysInMo THEN
         dd = dd - daysInMo: INCR mo
         IF mo > 12 THEN mo = mo - 12: INCR yr
      END IF
   END IF
   sMonth = RIGHT$("0" + TRIM$(STR$(mo)), 2)
   sDay = RIGHT$("0" + TRIM$(STR$(dd)), 2)
   sYear = TRIM$(STR$(yr))
   IF noDays THEN
      FUNCTION = sMonth + "/" + sYear
   ELSE
      FUNCTION = sMonth + "/" + sDay + "/" + sYear
   END IF
END FUNCTION

SUB showDateFmt(nCtrl AS LONG)
   LOCAL s1, s2, s3, s, sCtrl, month AS STRING
   LOCAL k1, k2, m, n AS LONG
   month = "Jan, Feb, Mar, Apr, May, Jun, Jul, Aug, Sep, Oct, Nov, Dec"
   CONTROL GET TEXT hMain, nCtrl TO sCtrl
   nErr = 0: processDate sCtrl: IF nErr THEN EXIT SUB
   CONTROL GET CHECK hMain, %IDC_Format TO k1
   CONTROL GET CHECK hMain, %IDC_WkDay TO k2
   sCtrl = TRIM$(PARSE$(sCtrl, "[", 1)) 'remove weekday
   IF k1 THEN
      REPLACE "-" WITH "/" IN sCtrl
      m = PARSECOUNT(sCtrl, "/")
      n = VAL(PARSE$(sCtrl, "/", 1))
      IF n < 1 OR n > 12 THEN nErr = 1: EXIT SUB
      s = TRIM$(PARSE$(month, ",", n))
      IF m = 2 THEN
         sCtrl = s + " " + PARSE$(sCtrl, "/", 2)
      ELSEIF m = 3 THEN
         s2 = PARSE$(sCtrl, "/", 2): s2 = LTRIM$(s2, "0")
         s3 = PARSE$(sCtrl, "/", 3)
         IF VAL(s2) > daysInMonth(n, VAL(s3)) THEN nErr = 2: EXIT SUB
         IF VAL(s3) < 1600 OR VAL(s3) > 2100 THEN nErr = 3: EXIT SUB
         sCtrl = s + " " + s2 + ", " + s3
      END IF
      IF k2 THEN s = sCtrl: sCtrl = sCtrl + addWeekDay(s)
      CONTROL SET TEXT hMain, nCtrl, sCtrl
   ELSE
      s = LEFT$(sCtrl, 3): n = INSTR(LCASE$(month), LCASE$(s))
      IF n THEN
         REPLACE ", " WITH " " IN sCtrl
         REPLACE "," WITH " " IN sCtrl
         REPLACE " " WITH "/" IN sCtrl
         REPLACE PARSE$(sCtrl, "/", 1) WITH TRIM$(STR$((n-1)\5+1)) IN sCtrl
      END IF
      IF PARSECOUNT(sCtrl, ANY "/-") = 3 THEN
         s1 = PARSE$(sCtrl, ANY "/-", 1): s2 = PARSE$(sCtrl, ANY "/-", 2): s3 = PARSE$(sCtrl, ANY "/-", 3)
         IF VAL(s1) < 1 OR VAL(s1) > 12 THEN nErr = 1: EXIT SUB
         IF VAL(s2) > daysInMonth(VAL(s1), VAL(s3)) THEN nErr = 2: EXIT SUB
         IF VAL(s3) < 1600 OR VAL(s3) > 2100 THEN nErr = 3: EXIT SUB
         s1 = RIGHT$("0" + s1, 2): s2 = RIGHT$("0" + s2, 2): sCtrl = s1 + "/" + s2 + "/" + s3
      END IF
      IF k2 THEN s = sCtrl: sCtrl = sCtrl + addWeekDay(s)
      CONTROL SET TEXT hMain, nCtrl, sCtrl
   END IF
END SUB


FUNCTION addWeekDay(s AS STRING) AS STRING
   LOCAL n, mo, dd, yr AS LONG, ss, day AS STRING
   day = "Su,Mo,Tu,We,Th,Fr,Sa"
   processDate s
   IF nErr OR PARSECOUNT(s, "/") < 3 THEN EXIT FUNCTION
   mo = VAL(LEFT$(s, 2)): dd = VAL(MID$(s, 4, 2)): yr = VAL(RIGHT$(s, 4))
   n = dayOfWeek(mo, yr, dd): ss = PARSE$(day, n + 1)
   IF LEN(ss) THEN FUNCTION = "  [" + ss + "]"
END FUNCTION

FUNCTION calendar(n AS LONG) AS STRING
   LOCAL nStyle, fcn AS LONG
   LOCAL s AS STRING
   DIALOG NEW 0, "", , , 470, 370, %WS_SYSMENU TO hCal
   DIALOG SET COLOR hCal, %BLACK, %WHITE
   CONTROL ADD LABEL, hCal, 201, "Century: ", 100, 5, 50, 10, %SS_RIGHT
   CONTROL ADD LABEL, hCal, 202, "Year: ", 210, 5, 50, 10, %SS_RIGHT
   CONTROL SET COLOR hCal, 201, %BLACK, %WHITE
   CONTROL SET COLOR hCal, 202, %BLACK, %WHITE
   nStyle = %CBS_DROPDOWNLIST OR %WS_VSCROLL
   CONTROL ADD COMBOBOX, hCal, 101, , 150, 5, 60, 60, nStyle
   nStyle = %CBS_DROPDOWN OR %WS_VSCROLL
   CONTROL ADD COMBOBOX, hCal, 102, , 260, 5, 60, 224, nStyle
   DIALOG SET USER hCal, 1, n
   DIALOG SHOW MODAL hCal CALL calendarProc TO fcn
   IF fcn THEN
      s = TRIM$(STR$(fcn))
      FUNCTION = MID$(s, 5, 2) + "/" + RIGHT$(s, 2) + "/" + LEFT$(s, 4)
   END IF
END FUNCTION

CALLBACK FUNCTION calendarProc
   'LOCAL i, j, k, m, n, x, y, mo, mm, nn, startDay, days AS LONG
   LOCAL i, n, x, y AS LONG
   LOCAL s AS STRING, t AS SIZEL, sz AS ASCIIZ * 5, ps AS PAINTSTRUCT
   STATIC hOrigFont, century, year, month, day AS LONG
   SELECT CASE CBMSG
      CASE %WM_INITDIALOG
         REDIM months(12) AS STRING
         months(1) = "January" : months(2) = "February" : months(3) = "March"
         months(4) = "April"   : months(5) = "May"      : months(6) = "June"
         months(7) = "July"    : months(8) = "August"   : months(9) = "September"
         months(10)= "October" : months(11)= "November" : months(12)= "December"
         FOR i = 1600 TO 2000 STEP 100
            COMBOBOX ADD CBHNDL, 101, TRIM$(STR$(i))
         NEXT i
         CONTROL SET FOCUS CBHNDL, 102
         hdc = GetDC(CBHNDL)
         hFont = makeFont(hdc, "Courier New", 9)
         hFont1 = makeFont(hDC, "MS Sans Serif", 10, "b")
         hOrigFont = SelectObject(hdc, hFont)
         sz = SPACE$(3):  GetTextExtentPoint32 hdc, sz, LEN(sz), t
         lenDay = t.cx
         DIALOG GET USER CBHNDL, 1 TO n: s = TRIM$(STR$(n))
         IF n THEN
            year = VAL(LEFT$(s, 4)): month = VAL(MID$(s, 5, 2)): day = VAL(RIGHT$(s, 2))
            century = (year \ 100) * 100: n = year \ 100 - 15
            COMBOBOX SELECT CBHNDL, 101, n: s = TRIM$(STR$(year))
            CONTROL SET TEXT CBHNDL, 102, s
            DIALOG SET TEXT CBHNDL, s + " Calendar"
         ELSE
            year = VAL(RIGHT$(DATE$, 4)): month = 0: day = 0
            DIALOG SET TEXT CBHNDL, TRIM$(STR$(year)) + " Calendar"
         END IF
      CASE %WM_PAINT
         BeginPaint CBHNDL, ps
         showMonths year, month, day
         EndPaint CBHNDL, ps
      CASE %WM_COMMAND
         IF CBCTL = 102 THEN
            IF CBCTLMSG = %CBN_SETFOCUS THEN
               CONTROL SEND CBHNDL, 101, %CB_GETCURSEL, 0, 0 TO n
               CONTROL SEND CBHNDL, 101, %CB_GETLBTEXT, n, VARPTR(sz)
               IF century > 0 THEN
                  COMBOBOX RESET CBHNDL, 102
                  FOR i = century TO century + 99
                     COMBOBOX ADD CBHNDL, 102, TRIM$(STR$(i))
                  NEXT i
                  COMBOBOX SELECT CBHNDL, 102, year - century + 1
               END IF
            ELSEIF CBCTLMSG = %CBN_SELENDOK THEN
               CONTROL SEND CBHNDL, CBCTL, %CB_GETCURSEL, 0, 0 TO n
               CONTROL SEND CBHNDL, CBCTL, %CB_GETLBTEXT, n, VARPTR(sz)
               year = VAL(sz)
               DIALOG SET TEXT CBHNDL, TRIM$(STR$(year)) + " Calendar"
               InvalidateRect CBHNDL, BYVAL 0, 1
            ELSEIF CBCTLMSG = %CBN_EDITCHANGE THEN
               COMBOBOX GET TEXT CBHNDL, CBCTL TO s
               IF LEN(s) = 0 THEN
                  CONTROL GET TEXT CBHNDL, CBCTL TO s
               END IF
               IF VAL(s) < 1600 OR VAL(s) > 2100 THEN EXIT FUNCTION
               year = VAL(s): n = (year \ 100) * 100: sz = TRIM$(STR$(n))
               CONTROL SEND CBHNDL, 101, %CB_SELECTSTRING, -1, VARPTR(sz)
               DIALOG SET TEXT CBHNDL, TRIM$(STR$(year)) + " Calendar"
               InvalidateRect CBHNDL, BYVAL 0, 1
            END IF
         ELSEIF CBCTL = 101 THEN
            CONTROL SEND CBHNDL, CBCTL, %CB_GETCURSEL, 0, 0 TO n
            CONTROL SEND CBHNDL, CBCTL, %CB_GETLBTEXT, n, VARPTR(sz)
            century = VAL(sz): EXIT FUNCTION
            IF century < 1600 AND century > 2100 THEN EXIT FUNCTION
         END IF
      CASE %WM_LBUTTONDOWN
         x = LOWRD(CBLPARAM): y = HIWRD(CBLPARAM)
         IF xCal AND yCal THEN hiLiteDay year, xCal, yCal, 0
         calDay = 1: hiLiteDay year, x, y, calDay
      CASE %WM_LBUTTONUP
         IF calDay THEN
            SLEEP 100: DIALOG END CBHNDL, calDay
         END IF
      CASE %WM_DESTROY
         SelectObject hdc, hOrigFont
         DeleteObject hFont
         DeleteObject hFont1
         ReleaseDC hCal, hdc
      CASE %WM_SYSCOMMAND
         DIALOG END CBHNDL, 0
   END SELECT
END FUNCTION

SUB showMonths(yr AS LONG, month AS LONG, day AS LONG)
   LOCAL i, j, k, m, n, mm, nn, startDay, days, mo AS LONG
   LOCAL s, sDay, ti AS STRING, t AS SIZEL, sz AS ASCIIZ * 128
   top = 50 'top of calendar months
   ti = "Su Mo Tu We Th Fr Sa"
   FOR j = 0 TO 2
   FOR i = 0 TO 3
      m = 170 * j: n = 180 * i: INCR mo
      s = months(mo): sz = s
      GetTextExtentPoint32 hdc, sz, LEN(sz), t
      SelectObject hdc, hFont1
      showText hdc, 75 + n - (t.cx)/2, top + 10 + 170*j, s  'specify month
      SelectObject hdc, hFont
      showText hdc, 7 + n, top + 30 + m, ti  'specify Su Mo Tu We Th Fr Sa
      MoveToEx hDC, 7 + n, top + 45 + m, BYVAL %NULL 'draw horiz line
      LineTo hDC, 150 + n, top + 45 + m
      IF i < 3 THEN 'draw vert lines
         MoveToEx hDC, 170 + n, top + 20 + m, BYVAL %NULL
         LineTo hDC, 170 + n, top + 180 + m
      END IF
      startDay = dayOfWeek(mo, yr, 1)
      days = daysInMonth(mo, yr)
      m = 0: n = startDay: s = SPACE$(3 * n)
      FOR k = 1 TO days
         s = s + RIGHT$("  " + STR$(k), 3): INCR n
         IF n = 7 OR k = days THEN
            INCR m: nn = 7 + 180 * i: mm = top + 30 + 170*j + 20*m
            SetTextColor hdc, %BLACK
            SetBkColor hdc, %WHITE
            showText hDc, nn, mm, MID$(s, 2)
            sDay = RIGHT$("  " + TRIM$(STR$(day)), 3)
            n = INSTR(s, sDay)
            IF mo = month AND n THEN 'k = day THEN
               xCal = nn - lenDay/3 + n/3 * lenDay: yCal = mm
               hiLiteDay yr, xCal, yCal, 1
            END IF
            n = 0: s = ""
         END IF
      NEXT k
   NEXT i
   NEXT j
END SUB

FUNCTION dayOfWeek(month AS LONG, year AS LONG, day AS LONG) AS LONG
   'The AstroDay function defined here is a modified version of the Julian date formula used by astronomers.
   'http://support.microsoft.com/kb/q109451/
   LOCAL astroDay AS LONG, y AS DOUBLE
   y = year + (month - 2.85) / 12
   astroDay = INT(INT(INT(367*y)-1.75*INT(y)+day) -0.75*INT(0.01*y))+1721119
   FUNCTION = (astroDay - 3) MOD 7
END FUNCTION

SUB showText(hDC AS LONG, x AS LONG, y AS LONG, txt AS STRING)
   TextOut hDC, x, y, BYVAL STRPTR(txt), LEN(txt)
END SUB

FUNCTION MakeFont(hDC AS LONG, sFont AS STRING, BYVAL ptSize AS LONG, _
                  OPT BYVAL descr AS STRING) AS LONG
   'specify descr = b for bold, u for underline, i for italic
   LOCAL CyPixels AS LONG, charHt AS LONG, wt AS DWORD
   LOCAL bold AS DWORD, underline AS DWORD, italic AS DWORD
   IF LEN(descr) THEN
      BOLD = INSTR(descr, "b") + 300
      underline = INSTR(descr,"u")
      italic = INSTR(descr,"i")
   END IF
   CyPixels  = GetDeviceCaps(hDC, %LOGPIXELSY)
   charHt = (PtSize * CyPixels) \ 72
   wt = %FW_NORMAL + BOLD
   FUNCTION = CreateFont(-charHt, 0, 0, 0, wt, italic, underline, 0, _
              %ANSI_CHARSET, %OUT_TT_PRECIS, %CLIP_DEFAULT_PRECIS, _
              %DEFAULT_QUALITY, %FF_DONTCARE, BYCOPY sFont)
END FUNCTION

SUB hiLiteDay(yr AS LONG, x AS LONG, y AS LONG, calDay AS LONG)
   'calDay = 0  black on white (normal)
   '       = 1  white on blue (hiLite)
   'return calDay on exit
   LOCAL i, j, k, m, n, mo, mm, nn, startDay, days AS LONG
   i = (x - 7) \ 180: j = (y - 30 - top) \ 170
   mo = 4 * j + i + 1
   startDay = dayOfWeek(mo, yr, 1)
   days = daysInMonth(mo, yr)
   m = (y - 30 - top - 170 * j) \ 20
   n = (x - 7 - 180 * i + lenDay / 3) \ lenDay + 1
   k = 7 * (m - 1) + n - startDay
   IF k < 1 OR k > days OR mo < 1 OR mo > 12 THEN EXIT SUB
   nn = 7 + 180 * i + (n - 1) * lenDay
   mm = top + 30 + 170*j + 20*m
   IF calDay THEN
      SetTextColor hdc, %WHITE
      SetBkColor hdc, %BLUE
   ELSE
      SetTextColor hdc, %BLACK
      SetBkColor hdc, %WHITE
   END IF
   showText hdc, nn, mm, MID$(RIGHT$("  " + STR$(k), 3), 2)
   calDay = 10000 * yr + 100 * mo + k
END SUB

SUB Clipboard_SetText(BYVAL txt AS STRING)
   'copy txt to clipboard
   '-------------------------------------------------------------
   LOCAL lpMem AS ASCIIZ PTR
   LOCAL hMem  AS DWORD
   ' Allocate global memory block
   hMem  = GlobalAlloc(%GHND, LEN(txt) + 1)
   ' lock it and get pointer to memory location
   lpMem = GlobalLock(hMem)
   ' copy text into memory object
   @lpMem = txt
   ' unlock the memory object
   GlobalUnlock hMem
   ' add text to the clipboard
   OpenClipboard 0
   EmptyClipboard
   SetClipboardData %CF_TEXT, hMem
   CloseClipboard
END SUB
